@@ -3,18 +3,23 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory.
 
-__version__ = '1.1.2'
+__version__ = '1.1.10'
 
 __all__ = [
     'cmaes',
     'cmaescpp',
     'dacpp',
     'decpp',
+    'gcldecpp',
+    'ldecpp',
+    'lgcldecpp',
     'hhcpp',
     'retry',
     'advretry',
+    'multiretry',
     'pygmoretry',
     'optimizer',
     'astro',
+    'evaluator',
     'testfun',
 ]
