@@ -26,7 +26,7 @@ def minimize(fun,
              popsize = None, 
              max_evaluations = 100000, 
              stop_fittness = None, 
-             keep = 200.0,
+             keep = 30.0,
              f = 0.5,
              cr = 0.9,
              rg = Generator(MT19937()),
@@ -50,6 +50,8 @@ def minimize(fun,
             1. Instance of the `scipy.Bounds` class.
             2. Sequence of ``(min, max)`` pairs for each element in `x`. None
                is used to specify no bound.
+    popsize : int, optional
+        Population size.
     max_evaluations : int, optional
         Forced termination after ``max_evaluations`` function evaluations.
     stop_fittness : float, optional 
